package org.foxarmy.barcodescannerforemployees

import android.content.ContentValues
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.fragment.app.Fragment
import com.google.mlkit.vision.barcode.common.Barcode
import com.google.mlkit.vision.codescanner.GmsBarcodeScannerOptions
import com.google.mlkit.vision.codescanner.GmsBarcodeScanning
import org.foxarmy.barcodescannerforemployees.databinding.AddProductFragmentBinding
import java.io.File
import java.nio.file.Files
import java.nio.file.StandardCopyOption


class AddProductFragment : Fragment() {

    private var _binding: AddProductFragmentBinding? = null
    private lateinit var pictureFile: File
    private val binding get() = _binding!!
    private lateinit var picturesPath: File

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        picturesPath = File(requireContext().filesDir, "pictures")
        picturesPath.mkdirs()

        _binding = AddProductFragmentBinding.inflate(inflater, container, false)
        return binding.root
    }

    @RequiresApi(Build.VERSION_CODES.O)
    val takePicture = registerForActivityResult(ActivityResultContracts.TakePicture()) { success: Boolean ->
        if (success) {
            //Move picture to a proper directory according to its calculated hash
            val tempfile = File(requireContext().filesDir, "image.png")
            val imageHash = tempfile.inputStream().readBytes().toString(Charsets.UTF_8).md5()
            pictureFile = File(picturesPath, "$imageHash.png")
            Files.move(tempfile.toPath(), pictureFile.toPath(), StandardCopyOption.REPLACE_EXISTING)
            tempfile.delete()
            binding.imageView.setImageURI(getImageUri(requireActivity(), pictureFile))
            Log.i("QWERTYUIOP", "Picture saved")
        } else {
            Log.e("QWERTYUIOP", "Cannot save a picture")
        }
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun getPicture () {
        //Saving picture to a temp file for further hash calculation and moving to a proper directory
        val imageFile = File(requireContext().filesDir, "image.png")
        val imageUri = getImageUri(requireActivity(), imageFile)
        if (imageUri != null) {
            Log.d("QWERTYUIOP", imageUri.path.toString())
        }
        takePicture.launch(imageUri)
    }

    @RequiresApi(Build.VERSION_CODES.O)
    val requestPermissionLauncher =
        registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted: Boolean ->
            if (isGranted) {
                Toast.makeText(requireContext(), "Granted", Toast.LENGTH_LONG).show()
                getPicture()
            } else {
                Toast.makeText(requireContext(), "I need permission in order to take a picture", Toast.LENGTH_LONG).show()
            }
        }

    @RequiresApi(Build.VERSION_CODES.R)
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.saveButton.setOnClickListener {
            val productName = binding.productName.text.toString()
            val netWeight = binding.netWeight.text
            if (!this::pictureFile.isInitialized || !pictureFile.exists()) {
                Toast.makeText(context, "Please, make a picture of a product!", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (productName == "") {
                Toast.makeText(context, "Please, write a name of a product!", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            if (netWeight.toString() == "" || netWeight.toString().toDoubleOrNull() == null) {
                Toast.makeText(context, "Please, write a valid net weight of a product!", Toast.LENGTH_SHORT).show()
            }

            val db = DBStorageController(requireContext()).writableDatabase
            Log.d("QWERTYUIOP", "Putting ${pictureFile.name}")
            val values = ContentValues().apply {
                put(ProductContract.ProductEntry.PRODUCT_NAME, productName)
                put(ProductContract.ProductEntry.PRODUCT_NET_WEIGHT, netWeight.toString())
                put(ProductContract.ProductEntry.IMAGE_FILENAME, pictureFile.name)
            }

            db.insert(ProductContract.ProductEntry.TABLE_NAME, null, values)
        }

        binding.takePictureButton.setOnClickListener {

            requestPermissionLauncher.launch(android.Manifest.permission.CAMERA)

//            val imageUri: Uri = Uri.fromFile(File(File(context?.filesDir, "pictures"), "test.jpg"))
//            val imageUri = Uri.parse("pictures/test.jpg")
//            val picturesDir = File(context?.filesDir, "pictures")
//            picturesDir.mkdirs()
//            val uri = Uri.fromFile(File(picturesDir, "test.jpg"))
//            Log.d("", uri.toString())
//            takePicture.launch(uri)
        }

        binding.scanButton.setOnClickListener {
            val options = GmsBarcodeScannerOptions.Builder()
                .setBarcodeFormats(
                    Barcode.FORMAT_EAN_13
                )
                .build()
            val scanner = GmsBarcodeScanning.getClient(requireContext())
            scanner.startScan()
                .addOnSuccessListener { barcode ->
                    binding.productName.setText(barcode.rawValue)

//                    val url = URL()
                }
                .addOnFailureListener { e ->
                    Toast.makeText(
                        requireContext(),
                        "Failed to scan barcode. Please, try again or enter data manually",
                        Toast.LENGTH_LONG
                    ).show()
                }
        }

    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}